"use strict";

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var _require = require('styled-components'),
    css = _require.css;

var _require2 = require('./errors'),
    BreakpointNotFoundError = _require2.BreakpointNotFoundError,
    NextBreakpointNotFoundError = _require2.NextBreakpointNotFoundError;

module.exports =
/*#__PURE__*/
function () {
  function Media(sizes) {
    _classCallCheck(this, Media);

    this.sizes = sizes;
  }

  _createClass(Media, [{
    key: "_isBreakpoint",
    value: function _isBreakpoint(breakpoint) {
      return Boolean(this.sizes[breakpoint]);
    }
  }, {
    key: "_next",
    value: function _next(breakpoint) {
      var keys = Object.keys(this.sizes);
      var index = keys.indexOf(breakpoint);
      return keys[index + 1];
    }
  }, {
    key: "up",
    value: function up(breakpoint) {
      var _this = this;

      return function () {
        if (!_this._isBreakpoint(breakpoint)) {
          throw new BreakpointNotFoundError(breakpoint);
        }

        return css(["@media (min-width:", "px){", "}"], _this.sizes[breakpoint], css.apply(void 0, arguments));
      };
    } // The max-width value is calculated as the next breakpoint less 0.02px.
    // See https://www.w3.org/TR/mediaqueries-4/#mq-min-max
    // Uses 0.02px rather than 0.01px to work around a current rounding bug
    // in Safari. See https://bugs.webkit.org/show_bug.cgi?id=178261

  }, {
    key: "down",
    value: function down(breakpoint) {
      var _this2 = this;

      return function () {
        if (!_this2._isBreakpoint(breakpoint)) {
          throw new BreakpointNotFoundError(breakpoint);
        }

        var nextBreakpoint = _this2._next(breakpoint);

        if (!_this2._isBreakpoint(nextBreakpoint)) {
          throw new NextBreakpointNotFoundError(breakpoint);
        }

        return css(["@media (max-width:", "px){", "}"], _this2.sizes[nextBreakpoint] - 0.02, css.apply(void 0, arguments));
      };
    }
  }, {
    key: "between",
    value: function between(min, max) {
      var _this3 = this;

      return function () {
        if (!_this3._isBreakpoint(min)) {
          throw new BreakpointNotFoundError(min);
        }

        if (!_this3._isBreakpoint(max)) {
          throw new BreakpointNotFoundError(max);
        }

        return css(["@media ( min-width:", "px ) and ( max-width:", "px ){", "}"], _this3.sizes[min], _this3.sizes[max] - 0.02, css.apply(void 0, arguments));
      };
    }
  }, {
    key: "only",
    value: function only(breakpoint) {
      var nextBreakpoint = this._next(breakpoint);

      return this._isBreakpoint(nextBreakpoint) ? this.between(breakpoint, nextBreakpoint) : this.up(breakpoint);
    }
  }]);

  return Media;
}();